package ru.dbotthepony.mc.otm.player.android

import net.minecraft.network.chat.Component
import net.minecraft.network.chat.ComponentContents
import net.minecraft.network.chat.MutableComponent
import net.minecraft.network.chat.contents.TranslatableContents
import ru.dbotthepony.mc.otm.player.MatteryPlayer
import ru.dbotthepony.mc.otm.core.getKeyNullable
import ru.dbotthepony.mc.otm.registry.MBuiltInRegistries
import ru.dbotthepony.mc.otm.registry.MRegistry

open class AndroidFeatureType<T : AndroidFeature> {
	protected val factory: (AndroidFeatureType<T>, MatteryPlayer) -> T

	constructor(factory: (MatteryPlayer) -> T) : super() {
		this.factory = { _, capability -> factory.invoke(capability) }
	}

	constructor(factory: (AndroidFeatureType<T>, MatteryPlayer) -> T) : super() {
		this.factory = factory
	}

	fun create(android: MatteryPlayer): T {
		return factory.invoke(this, android)
	}

	open fun isApplicable(android: MatteryPlayer) = true

	val registryName by lazy {
		MBuiltInRegistries.ANDROID_FEATURE.getKeyNullable(this)
	}

	open val displayContents: ComponentContents by lazy {
		TranslatableContents(displayId, null, arrayOf())
	}

	open val displayName: Component by lazy {
		MutableComponent.create(displayContents)
	}

	open val displayId: String by lazy {
		registryName?.let { "android_feature.${it.namespace}.${it.path}" } ?: "android_feature.null.null"
	}
}
